<?php

namespace Drupal\stacked_permissions\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\user\Entity\User;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Ajax\RemoveCommand;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\stacked_permissions\Entity\PermissionGroup;

class PermissionGroupController extends ControllerBase {
    public function editForm(PermissionGroup $permission_group) {
        // Load the permission group
        $form = $this->formBuilder()->getForm('Drupal\stacked_permissions\Form\PermissionGroupForm', $permission_group);
        return $form;
    }

    public function removeMember($permission_group, $user) {
        $response = new AjaxResponse();

        // Load the membership entity to delete.
        $membership_storage = \Drupal::entityTypeManager()->getStorage('permission_group_membership');
        $memberships = $membership_storage->loadByProperties([
            'group_id' => $permission_group,
            'user_id' => $user,
        ]);

        foreach ($memberships as $membership) {
            $membership->delete();
        }

        // Remove the member's row from the table.
        $response->addCommand(new RemoveCommand('.member-row-' . $user));

        // Render messages and add them to the custom message container.
        \Drupal::messenger()->addStatus($this->t('Member successfully removed from Permission Group.'));
        $messages = ['#type' => 'status_messages'];
        $status_messages = \Drupal::service('renderer')->renderRoot($messages);
        $response->addCommand(new HtmlCommand('#custom-messages-remove', $status_messages));

        return $response;
    }


    /**
     * Autocomplete callback for users.
     */
    public function userAutocomplete(Request $request, $permission_group) {
        $matches = [];
        $string = $request->query->get('q');

        if ($string) {
            // Load the permission group to get current members
            $group = \Drupal::entityTypeManager()->getStorage('permission_group')->load($permission_group);
            $current_member_ids = [];

            if ($group) {
                // Load current members
                $membership_storage = \Drupal::entityTypeManager()->getStorage('permission_group_membership');
                $memberships = $membership_storage->loadByProperties(['group_id' => $group->id()]);

                foreach ($memberships as $membership) {
                    $current_member_ids[] = $membership->get('user_id')->target_id;
                }
            }

            // Query for users matching the input string
            $query = \Drupal::entityQuery('user')
                ->condition('name', '%' . $string . '%', 'LIKE')
                ->accessCheck(TRUE)
                ->range(0, 10);

            // Only add the NOT IN condition if there are current members
            if (!empty($current_member_ids)) {
                $query->condition('uid', $current_member_ids, 'NOT IN');
            }

            try {
                $user_ids = $query->execute();
                $users = User::loadMultiple($user_ids);

                foreach ($users as $user) {
                    $matches[] = [
                        'value' => $user->id(), // Use the user ID as the value for backend processing
                        'label' => $user->getDisplayName(), // Only show the username in the suggestions
                    ];
                }
            } catch (\Exception $e) {
                \Drupal::logger('stacked_permissions')->error('User autocomplete error: @message', ['@message' => $e->getMessage()]);
            }
        }

        return new JsonResponse($matches);
    }
}
