<?php

namespace Drupal\stacked_permissions\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\ContentEntityInterface;

/**
 * @ContentEntityType(
 *   id = "permission_group",
 *   label = @Translation("Permission Group"),
 *   base_table = "permission_group",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "name",
 *     "uuid" = "uuid"
 *   },
 *   handlers = {
 *     "storage" = "Drupal\Core\Entity\Sql\SqlContentEntityStorage",
 *     "view_builder" = "Drupal\stacked_permissions\PermissionGroupViewBuilder",
 *     "views_data" = "Drupal\stacked_permissions\PermissionGroupViewsData",
 *     "form" = {
 *       "default" = "Drupal\stacked_permissions\Form\PermissionGroupForm",
 *       "add" = "Drupal\stacked_permissions\Form\PermissionGroupAddForm",
 *       "delete" = "Drupal\stacked_permissions\Form\PermissionGroupDeleteForm"
 *     },
 *     "list_builder" = "Drupal\stacked_permissions\PermissionGroupListBuilder",
 *     "access" = "Drupal\stacked_permissions\PermissionGroupAccessControlHandler"
 *   },
 *   admin_permission = "administer site configuration",
 *   field_ui_base_route = "",
 *   links = {
 *     "canonical" = "/permission-group/{permission_group}",
 *     "add-form" = "/permission-group/add",
 *     "edit-form" = "/permission-group/{permission_group}/edit",
 *     "delete-form" = "/permission-group/{permission_group}/delete",
 *     "collection" = "/permission-group"
 *   }
 * )
 */

class PermissionGroup extends ContentEntityBase implements ContentEntityInterface {
    public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
        $fields = parent::baseFieldDefinitions($entity_type);

        $fields['name'] = BaseFieldDefinition::create('string')
            ->setLabel(t('Permission Group Name'))
            ->setRequired(TRUE)
            ->setSettings([
                'max_length' => 255,
            ])
            ->setDisplayOptions('view', ['label' => 'inline', 'type' => 'string'])
            ->setDisplayOptions('form', ['type' => 'string_textfield']);

        $fields['created'] = BaseFieldDefinition::create('created')
            ->setLabel(t('Created'))
            ->setDescription(t('The time that the permission group was created.'));

        $fields['changed'] = BaseFieldDefinition::create('changed')
            ->setLabel(t('Updated'))
            ->setDescription(t('The time that the permission group was last updated.'));

        // Define a field to store permissions.
        $fields['permissions'] = BaseFieldDefinition::create('string_long')
            ->setLabel(t('Permissions'))
            ->setDescription(t('Serialized array of permissions assigned to the group.'))
            ->setDefaultValue('');

        return $fields;
    }
}
