<?php

namespace Drupal\stacked_permissions\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\user\UserInterface;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Defines the Permission Group Membership entity.
 *
 * @ContentEntityType(
 *   id = "permission_group_membership",
 *   label = @Translation("Permission Group Membership"),
 *   base_table = "permission_group_membership",
 *   entity_keys = {
 *     "id" = "id",
 *     "uuid" = "uuid",
 *   },
 *   handlers = {
 *     "list_builder" = "Drupal\stacked_permissions\PermissionGroupMembershipListBuilder",
 *     "views_data" = "Drupal\stacked_permissions\PermissionGroupMembershipViewsData",
 *     "form" = {
 *       "default" = "Drupal\stacked_permissions\Form\PermissionGroupMembershipForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm"
 *     },
 *     "access" = "Drupal\stacked_permissions\PermissionGroupMembershipAccessControlHandler",
 *   },
 *   links = {
 *     "canonical" = "/permission-group-membership/{permission_group_membership}",
 *     "add-form" = "/permission-group-membership/add",
 *     "delete-form" = "/permission-group-membership/{permission_group_membership}/delete"
 *   },
 * )
 */
class PermissionGroupMembership extends ContentEntityBase {
    use EntityChangedTrait;

    public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
        $fields = parent::baseFieldDefinitions($entity_type);

        $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
            ->setLabel(t('User'))
            ->setSetting('target_type', 'user')
            ->setRequired(TRUE);

        $fields['group_id'] = BaseFieldDefinition::create('entity_reference')
            ->setLabel(t('Permission Group'))
            ->setSetting('target_type', 'permission_group')
            ->setRequired(TRUE);

        $fields['created'] = BaseFieldDefinition::create('created')
            ->setLabel(t('Created'));

        $fields['changed'] = BaseFieldDefinition::create('changed')
            ->setLabel(t('Changed'));

        return $fields;
    }
}
