<?php

namespace Drupal\stacked_permissions;

use Drupal\Core\Access\AccessManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Url;
use Drupal\Core\Link;

class PermissionGroupListBuilder extends EntityListBuilder {

    public function render(): array {
        $build = [];

        $access_control = \Drupal::entityTypeManager()
            ->getAccessControlHandler($this->entityTypeId);

        if ($this->entityType->hasLinkTemplate('add-form') && $access_control->createAccess(NULL, NULL, [], TRUE)->isAllowed()) {
            $url = Url::fromRoute("entity.{$this->entityTypeId}.add_form");
            $link = Link::fromTextAndUrl($this->t('Add Permission Group'), $url)->toRenderable();
            $link['#attributes']['class'][] = 'button button--primary';

            $build['add_link'] = [
                '#type' => 'container',
                '#attributes' => ['class' => ['permission-group-add-button']],
                'link' => $link,
            ];
        }

        $build += parent::render();
        return $build;
    }


    /**
     * {@inheritdoc}
     */
    public function buildHeader() {
        $header['id'] = $this->t('ID');
        $header['name'] = $this->t('Name');

        // Check if at least one entity allows edit or delete access
        $show_operations = FALSE;
        foreach ($this->load() as $entity) {
            if ($entity->access('edit') || $entity->access('delete')) {
                $show_operations = TRUE;
                break;
            }
        }

        if ($show_operations) {
            $header['operations'] = $this->t('Operations');
        }

        return $header;
    }

//    public function buildHeader() {
//        $header['id'] = $this->t('ID');
//        $header['name'] = $this->t('Name');
//        $header['operations'] = $this->t('Operations');
//        return $header;
//    }

    /**
     * {@inheritdoc}
     */
    public function buildRow(EntityInterface $entity) {
        $row['id'] = $entity->id();

        // Make the name a link to the canonical view route.
        $url = Url::fromRoute("entity.{$entity->getEntityTypeId()}.canonical", [
            $entity->getEntityTypeId() => $entity->id(),
        ]);
        $row['name'] = Link::fromTextAndUrl($entity->label(), $url)->toString();

        // Only add operations if the user has permission
        $operations = $this->buildOperations($entity);
        if (!empty($operations)) {
            $row['operations']['data'] = [
                '#type' => 'operations',
                '#links' => $operations,
            ];
        }

        return $row;
    }

//    public function buildRow(EntityInterface $entity) {
//        $row['id'] = $entity->id();
//        $row['name'] = $entity->label();
//
//        // Provide the edit/delete operations.
//        $row['operations']['data'] = [
//            '#type' => 'operations',
//            '#links' => $this->buildOperations($entity),
//        ];
//
//        return $row;
//    }

    /**
     * Returns an array of operation links.
     */

    public function buildOperations(EntityInterface $entity) {
        $operations = [];
        $entity_type_id = $entity->getEntityTypeId();

        if ($entity->access('edit')) {
            $operations['edit'] = [
                'title' => $this->t('Edit'),
                'url' => Url::fromRoute("entity.{$entity_type_id}.edit_form", [$entity_type_id => $entity->id()]),
            ];
        }

        if ($entity->access('delete')) {
            $operations['delete'] = [
                'title' => $this->t('Delete'),
                'url' => Url::fromRoute("entity.{$entity_type_id}.delete_form", [$entity_type_id => $entity->id()]),
            ];
        }

        return $operations;
    }
}
