<?php

namespace Drupal\stacked_permissions;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityViewBuilder;
use Drupal\Core\Database\Database;
use Drupal\user\Entity\User;

/**
 * View builder for PermissionGroup entity.
 */
class PermissionGroupViewBuilder extends EntityViewBuilder {

    /**
     * {@inheritdoc}
     */
    public function view(EntityInterface $entity, $view_mode = 'full', $langcode = NULL) {
        $build = parent::view($entity, $view_mode, $langcode);

        // Replace the title, if rendered via controller
        $build['#title'] = $entity->label();

        $build['group_members_view'] = [
            '#type' => 'details',
            '#title' => $this->t('Group Members'),
            '#open' => FALSE,
            'view' => views_embed_view('permission_group_membership', 'default', $entity->id()),
        ];

//        $connection = Database::getConnection();
//        $query = $connection->select('permission_group_membership', 'pgm')
//            ->fields('pgm', ['user_id'])
//            ->condition('group_id', $entity->id())
//            ->extend('Drupal\Core\Database\Query\PagerSelectExtender')
//            ->limit(50);
//
//        $user_ids = $query->execute()->fetchCol();
//        $users = User::loadMultiple($user_ids);
//
//        // Format display names
//        $member_list = [];
//        foreach ($users as $user) {
//            $member_list[] = $user->getDisplayName();
//        }
//
//        $build['members'] = [
//            '#type' => 'details',
//            '#title' => $this->t('Group Members'),
//            '#open' => FALSE,
//            'list' => [
//                '#theme' => 'item_list',
//                '#items' => $member_list,
//            ],
//            'pager' => [
//                '#type' => 'pager',
//            ],
//        ];

        // Deserialize permissions
        $serialized = $entity->get('permissions')->value;
        $permissions = unserialize($serialized) ?: [];

        $grouped_permissions = [];
        foreach ($permissions as $permission_id => $value) {
            if ($value) {
                $parts = explode('_|_', $permission_id);
                if (count($parts) === 2) {
                    $module = $parts[0];
                    $permission = str_replace('-_-', ' ', $parts[1]);

                    // Get module name
                    try {
                        $module_name = \Drupal::service('extension.list.module')->getName($module);
                    } catch (\Exception $e) {
                        $module_name = $module;
                    }

                    $grouped_permissions[$module_name][] = ucwords($permission);
                }
            }
        }

        $permissions_list = [];
        foreach ($grouped_permissions as $module_name => $module_permissions) {
            $permissions_list[] = [
                '#markup' => '<p><strong><u>' . $module_name . '</u></strong></p>',
            ];
            $permissions_list[] = [
                '#theme' => 'item_list',
                '#items' => $module_permissions,
            ];
            $permissions_list[] = [
                '#markup' => '<br><hr><br>',
            ];
        }

        if(empty($permissions_list)) {
            $permissions_list[] = [
                '#type' => 'container',
                '#attributes' => ['style' => 'text-align: center; padding: 20px 0;'],
                'content' => [
                    '#markup' => $this->t('No Granted Permissions Found'),
                ],
            ];
        }

        $build['permissions'] = [
            '#type' => 'details',
            '#title' => $this->t('Granted Permissions'),
            '#open' => FALSE,
            'grouped' => $permissions_list,
        ];

        return $build;
    }

}
