<?php

namespace Drupal\stacked_permissions;

use Drupal\views\EntityViewsData;
use Drupal\views\EntityViewsDataInterface;

/**
 * Provides the Views data for the Permission Group entity.
 */
class PermissionGroupViewsData extends EntityViewsData implements EntityViewsDataInterface {

    /**
     * {@inheritdoc}
     */
    public function getViewsData() {
        $data = parent::getViewsData();

        $data['permission_group']['name']['filter']['title'] = t('Group Name');

        // Expose the serialized permissions field (if needed raw)
        $data['permission_group']['permissions'] = [
            'title' => t('Raw Permissions'),
            'help' => t('Serialized array of permissions assigned to the group.'),
            'field' => [
                'id' => 'standard',
            ],
        ];

        // Add a readable permission field via a custom handler
        $data['permission_group']['readable_permissions'] = [
            'title' => t('Readable Permissions'),
            'help' => t('Calculated readable permission list.'),
            'field' => [
                'id' => 'readable_permissions_field',
            ],
        ];

        //Tie membership as a relationship
        $data['permission_group']['membership'] = [
            'title' => t('Permission Group Memberships'),
            'help' => t('Membership entities that reference this group.'),
            'relationship' => [
                'id' => 'standard',
                'base' => 'permission_group_membership',
                'base field' => 'group_id',
                'relationship field' => 'id',
                'label' => t('Memberships'),
            ],
        ];
//        $data['permission_group']['membership'] = [
//            'title' => t('Permission Group Memberships'),
//            'help' => t('The memberships belonging to this group.'),
//            'relationship' => [
//                'base' => 'permission_group_membership',
//                'base field' => 'group_id',
//                'relationship field' => 'id',
//                'id' => 'standard',
//                'field_name' => 'group_id',
//                'field table' => 'permission_group_membership',
//                'field field' => 'group_id',
//            ],
//        ];

        return $data;
    }

}