<?php

namespace Drupal\stacked_permissions\Plugin\views\field;

use Drupal\views\ResultRow;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\Annotation\ViewsField;
use Drupal\Component\Utility\Xss;

/**
 * @ViewsField("readable_permissions_field")
 */
class ReadablePermissionsField extends FieldPluginBase {

    public function usesGroupBy() {
        return FALSE;
    }

    public function query() {
        // Prevent Views from including this field in SQL SELECT.
        $this->field_alias = NULL;
    }

    public function render(ResultRow $values) {
        if (!isset($values->_entity)) {
            return '';
        }

        $entity = $values->_entity;
        $permissions = @unserialize($entity->get('permissions')->value ?? '') ?: [];

        $grouped = [];
        foreach ($permissions as $perm_id => $enabled) {
            if (!$enabled) continue;

            $parts = explode('_|_', $perm_id);
            if (count($parts) !== 2) continue;

            $module = $parts[0];
            $perm = str_replace('-_-', ' ', $parts[1]);

            $grouped[$module][] = $perm;
        }

        $output = '';
        foreach ($grouped as $module => $perms) {
            $readable_module = \Drupal::service('extension.list.module')->getName($module);
            $output .= '<strong>' . $readable_module . ':</strong> ' . ucwords(Xss::filter(implode(', ', $perms))) . '<br>';
        }

        return [
            '#type' => 'inline_template',
            '#template' => '{{ output|raw }}',
            '#context' => ['output' => $output],
        ];
    }
}
