<?php


namespace Drupal\stacked_permissions;

use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;

class StackedPermissionsAccountProxy implements AccountProxyInterface
{

    protected $original;
    protected $entityTypeManager;

    public function __construct(AccountProxyInterface $original, EntityTypeManagerInterface $entityTypeManager) {
        $this->original = $original;
        $this->entityTypeManager = $entityTypeManager;
    }

    protected function getGroupPermissions(AccountInterface $account)
    {
        $permissions = [];
        $membership_storage = $this->entityTypeManager->getStorage('permission_group_membership');
        $memberships = $membership_storage->loadByProperties(['user_id' => $account->id()]);

        foreach ($memberships as $membership) {
            $permission_group = $membership->get('group_id')->entity;

            if ($permission_group) {
                $group_permissions = unserialize($permission_group->get('permissions')->value) ?: [];

                foreach ($group_permissions as $permission => $value) {
                    if ($value) {
                        $permission = str_replace('-_-', ' ', explode('_|_', $permission)[1]);
                        $permissions[$permission] = $permission;
                    }
                }
            }
        }

        return $permissions;
    }

    public function hasPermission($permission)
    {
        if (in_array($permission, $this->getGroupPermissions($this->getAccount()))) {
            return TRUE;
        }

        // Fall back to original behavior
        return $this->original->hasPermission($permission);
    }

    // Delegate all other calls to the original service
    public function __call($method, $args)
    {
        return call_user_func_array([$this->original, $method], $args);
    }

    // AccountProxyInterface methods
    public function setAccount(AccountInterface $account)
    {
        $this->original->setAccount($account);
    }

    public function setInitialAccountId($account_id)
    {
        $this->original->setInitialAccountId($account_id);
    }

    // Required AccountInterface methods
    public function id()
    {
        return $this->original->id();
    }

    public function getRoles($exclude_locked_roles = FALSE)
    {
        return $this->original->getRoles($exclude_locked_roles);
    }

    public function hasPermissionExact($permission)
    {
        return $this->original->hasPermissionExact($permission);
    }

    public function isAuthenticated()
    {
        return $this->original->isAuthenticated();
    }

    public function isAnonymous()
    {
        return $this->original->isAnonymous();
    }

    public function getAccount()
    {
        return $this->original->getAccount();
    }

    public function getPreferredLangcode($fallback_to_default = TRUE)
    {
        return $this->original->getPreferredLangcode($fallback_to_default);
    }

    public function getPreferredAdminLangcode($fallback_to_default = TRUE)
    {
        return $this->original->getPreferredAdminLangcode($fallback_to_default);
    }

    public function getDisplayName()
    {
        return $this->original->getDisplayName();
    }

    public function getAccountName()
    {
        return $this->original->getAccountName();
    }

    public function getEmail()
    {
        return $this->original->getEmail();
    }

    public function getTimeZone()
    {
        return $this->original->getTimeZone();
    }

    public function getLastAccessedTime()
    {
        return $this->original->getLastAccessedTime();
    }

    public function isActive()
    {
        return $this->original->isActive();
    }

    public function getInitialAccountId()
    {
        return $this->original->getInitialAccountId();
    }

    public function getUuid()
    {
        return $this->original->getUuid();
    }

    public function getAccountType()
    {
        return $this->original->getAccountType();
    }

    public function isAnonymousAccessAllowed()
    {
        return $this->original->isAnonymousAccessAllowed();
    }

    public function get($property_name)
    {
        return $this->original->get($property_name);
    }
}


//namespace Drupal\stacked_permissions;
//
//use Drupal\Core\Session\AccountProxy;
//use Drupal\Core\Session\AccountInterface;
//use Drupal\Core\Entity\EntityTypeManagerInterface;
//use Symfony\Component\HttpFoundation\RequestStack;
//use Drupal\user\PermissionHandlerInterface;
//use Symfony\Component\HttpFoundation\Session\SessionInterface;
//
//class StackedPermissionsAccountProxy extends AccountProxy
//{
//    protected $entityTypeManager;
//
//    public function __construct(
//        SessionInterface           $session,
//        RequestStack               $request_stack,
//        PermissionHandlerInterface $permission_handler,
//        EntityTypeManagerInterface $entity_type_manager
//    )
//    {
//        parent::__construct($session, $request_stack, $permission_handler, $entity_type_manager);
//        $this->entityTypeManager = $entity_type_manager;
//    }
//
//    public function hasPermission($permission)
//    {
//        if (in_array($permission, $this->getGroupPermissions($this->getAccount()))) {
//            return TRUE;
//        }
//        return parent::hasPermission($permission);
//    }
//
//    protected function getGroupPermissions(AccountInterface $account)
//    {
//        $permissions = [];
//        $membership_storage = $this->entityTypeManager->getStorage('permission_group_membership');
//        $memberships = $membership_storage->loadByProperties(['user_id' => $account->id()]);
//
//        foreach ($memberships as $membership) {
//            $permission_group = $membership->get('group_id')->entity;
//
//            if ($permission_group) {
//                $group_permissions = unserialize($permission_group->get('permissions')->value) ?: [];
//
//                foreach ($group_permissions as $permission => $value) {
//                    if ($value) {
//                        $permission = str_replace('-_-', ' ', explode('_|_', $permission)[1]);
//                        $permissions[$permission] = $permission;
//                    }
//                }
//            }
//        }
//
//        Drupal::logger('stacked_permissions')->debug('User {uid} has additional permissions: @permissions', [
//            '@permissions' => implode(', ', array_keys($permissions)),
//            'uid' => $account->id(),
//        ]);
//
//        return $permissions;
//    }
//}